package com.zhaoliangsz.grainsearch.basic.config;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.baomidou.mybatisplus.core.injector.DefaultSqlInjector;
import com.baomidou.mybatisplus.core.injector.ISqlInjector;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import java.util.Date;

/**
 * @author : zyw
 * @date : 2023/8/17 15:33
 * @description: MybatisPlus配置类
 */
@Configuration
public class MybatisPlusConfig {
    /**
     * 逻辑删除
     * @return
     */
    @Bean
    public ISqlInjector sqlInjector(){
        return new DefaultSqlInjector();
    }

    /**
     * 字段自动填充
     */
    @ConditionalOnMissingBean(MetaObjectHandler.class)
    @Bean
    public MetaObjectHandler metaObjectHandler() {

        final String DEFAULT_USER = "sysadmin";

        final String CREATE_TIME_FIELD = "createTime";

        final String UPDATE_TIME_FIELD = "updateTime";

        final String UPDATE_BY_FIELD = "updateBy";

        final String CREATE_BY_FIELD = "createBy";

        final String CREATOR_ID_FIELD = "creatorId";

        final String HAS_DELETED_FIELD = "hasDeleted";

        return new MetaObjectHandler() {

            @Override
            public void insertFill(MetaObject metaObject) {
                Date currentDate = new Date();
                String currentUserId = null;
//                try{
//                    currentUserId = (String) SecurityUtils.getSubject().getPrincipal();
//                } catch (Exception e){
//                    currentUserId = RpcInvokeContext.getContext().getRequestBaggage("userId");
//                }
                this.setFieldValByName(CREATE_TIME_FIELD, currentDate, metaObject);
                this.setFieldValByName(UPDATE_TIME_FIELD, currentDate, metaObject);
                this.setFieldValByName(HAS_DELETED_FIELD, "0", metaObject);

                if (null != currentUserId) {
                    this.setFieldValByName(CREATE_BY_FIELD, currentUserId, metaObject);
                    this.setFieldValByName(UPDATE_BY_FIELD, currentUserId, metaObject);
                } else {
                    if (null == this.getFieldValByName(UPDATE_BY_FIELD, metaObject)) {
                        this.setFieldValByName(UPDATE_BY_FIELD, DEFAULT_USER, metaObject);
                    }

                    if (null == this.getFieldValByName(CREATE_BY_FIELD, metaObject)) {
                        this.setFieldValByName(CREATE_BY_FIELD, DEFAULT_USER, metaObject);
                    }
                }

//                this.setInternationResourceId(metaObject, false);
            }

            @Override
            public void updateFill(MetaObject metaObject) {
                String currentUserId = null;
//                try{
//                    currentUserId = (String) SecurityUtils.getSubject().getPrincipal();
//                } catch (Exception e){
//                    currentUserId = RpcInvokeContext.getContext().getRequestBaggage("userId");
//                }
                if (null != currentUserId) {
                    this.setFieldValByName(UPDATE_BY_FIELD, currentUserId, metaObject);
                } else if (null == this.getFieldValByName(UPDATE_BY_FIELD, metaObject)) {
                    this.setFieldValByName(UPDATE_BY_FIELD, DEFAULT_USER, metaObject);
                }
                this.setFieldValByName(UPDATE_TIME_FIELD, new Date(), metaObject);

//                this.setInternationResourceId(metaObject, true);
            }
        };
    }

}
