package com.zhaoliangsz.grainsearch.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zhaoliangsz.grainsearch.domain.entity.InventoryTable;
import com.zhaoliangsz.grainsearch.domain.entity.TestTable;
import com.zhaoliangsz.grainsearch.mapper.InventoryTableMapper;
import com.zhaoliangsz.grainsearch.mapper.TestTableMapper;
import com.zhaoliangsz.grainsearch.service.ITestTableService;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import com.zhaoliangsz.grainsearch.basic.exception.ServiceException;
import com.zhaoliangsz.grainsearch.basic.result.PageResult;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;

import java.sql.Wrapper;
import java.time.LocalDate;
import java.util.*;

import org.springframework.transaction.annotation.Transactional;
import com.zhaoliangsz.grainsearch.basic.exception.ExceptionDefinition;
/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zyw
 * @since 2023-08-21 14:40:08
 */
@Slf4j
@Service
public class TestTableServiceImpl extends ServiceImpl<TestTableMapper, TestTable> implements ITestTableService {
    @Autowired
    private TestTableMapper testTableMapper;
    @Autowired
    private InventoryTableMapper inventoryTableMapper;
    @Override
    public PageResult<TestTable> queryTestTableList(TestTable queryTestTable) throws ServiceException{
        try{
            Map<String,Object>pageView=new HashMap<>();
            pageView.put("startRecord",(queryTestTable.getCurrentPage()-1)
            *queryTestTable.getPageSize());
            pageView.put("pageSize",queryTestTable.getPageSize());
            pageView.put("currentPage",queryTestTable.getCurrentPage());
            Integer dataCount=baseMapper.getCountByCondition(pageView);
            List<TestTable>dataList=baseMapper.getListByCondition(pageView);
            return new PageResult<>(dataList==null?new ArrayList<>():dataList,dataCount==null?0:dataCount);
        }
        catch(Exception ex){
            log.error("[TestTableServiceImpl.queryTestTableList]异常:{}",ex.toString());
            throw new ServiceException(ExceptionDefinition.QUERY_DATA_EXCEPTION);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addTestTable(TestTable addTestTable) throws ServiceException {
        if (!"zlty".equals(addTestTable.getPassword())){
            throw new ServiceException(ExceptionDefinition.PASSWORD_ERROR);
        }
        else {
            //查询相同条件是否已添加过
            TestTable testTable=testTableMapper.selectOne(new QueryWrapper<TestTable>()
                    .eq("type",addTestTable.getType())
                    .eq("banci",addTestTable.getBanci())
                    .eq("banzu",addTestTable.getBanzu())
                    .eq("date",addTestTable.getDate()));
            boolean flag;
            InventoryTable inventoryTable=inventoryTableMapper.selectOne(new QueryWrapper<InventoryTable>()
                    .eq("date",addTestTable.getDate()));
            //编辑
            if (testTable!=null){
                addTestTable.setUpdateTime(new Date());
                addTestTable.setId(testTable.getId());
                LocalDate today = addTestTable.getDate();
                //入库
                if ("入库".equals(addTestTable.getType())){
                    inventoryTable.setInNum(addTestTable.getInNum());
                    //库存=当天库存减原入库加新入库
                    inventoryTable.setInventory(inventoryTable.getInventory()-testTable.getInNum()+addTestTable.getInNum());
                }
                //出库
                else{
                    //出库量先减原出库量再进行累加
                    inventoryTable.setOutNum(inventoryTable.getOutNum()-testTable.getTotal()+addTestTable.getTotal());
                    //库存=当天库存加原出库减新出库
                    inventoryTable.setInventory(inventoryTable.getInventory()+testTable.getTotal()-addTestTable.getTotal());
                }
                inventoryTable.setUpdateTime(new Date());
                inventoryTableMapper.updateById(inventoryTable);
                flag= this.updateById(addTestTable);
            }
            //新增
            else {
                addTestTable.setCreateTime(new Date());
                addTestTable.setUpdateTime(new Date());
                //获取本条数据日期
                LocalDate today = addTestTable.getDate();
                //获取前一天日期  查询前一天库存
                LocalDate yesterday = today.minusDays(1);
                InventoryTable inventoryTable1=inventoryTableMapper.selectOne(new QueryWrapper<InventoryTable>()
                        .eq("date",yesterday));
                //入库
                if ("入库".equals(addTestTable.getType())){
                    if (inventoryTable!=null){
                        inventoryTable.setInNum(addTestTable.getInNum());
                        //库存=当天库存加入库
                        inventoryTable.setInventory(inventoryTable.getInventory()+addTestTable.getInNum());
                        inventoryTable.setUpdateTime(new Date());
                        inventoryTableMapper.updateById(inventoryTable);
                    }
                    else{
                        InventoryTable inventoryTable2=new InventoryTable();
                        inventoryTable2.setCreateTime(new Date());
                        inventoryTable2.setUpdateTime(new Date());
                        inventoryTable2.setDate(addTestTable.getDate());
                        inventoryTable2.setInNum(addTestTable.getInNum());
                        //库存=前一天库存加入库
                        if (inventoryTable1!=null){
                            inventoryTable2.setInventory(inventoryTable1.getInventory()+addTestTable.getInNum());
                        }
                        else{
                            throw new ServiceException(ExceptionDefinition.NOT_YESTERDAY_INVENTORY);

                        }
                        inventoryTableMapper.insert(inventoryTable2);
                    }
                }
                //出库
                else{
                    if (inventoryTable!=null){
                        //出库量累加
                        inventoryTable.setOutNum(inventoryTable.getOutNum()+addTestTable.getTotal());
                        //库存=当天库存减出库
                        inventoryTable.setInventory(inventoryTable.getInventory()-addTestTable.getTotal());
                        inventoryTable.setUpdateTime(new Date());
                        inventoryTableMapper.updateById(inventoryTable);
                    }
                    else{
                        InventoryTable inventoryTable2=new InventoryTable();
                        inventoryTable2.setCreateTime(new Date());
                        inventoryTable2.setUpdateTime(new Date());
                        inventoryTable2.setDate(addTestTable.getDate());
                        inventoryTable2.setOutNum(addTestTable.getTotal());
                        //库存=前一天库存减出库
                        if (inventoryTable1!=null){
                            inventoryTable2.setInventory(inventoryTable1.getInventory()-addTestTable.getTotal());
                        }
                        else{
                            throw new ServiceException(ExceptionDefinition.NOT_YESTERDAY_INVENTORY);

                        }
                        inventoryTableMapper.insert(inventoryTable2);
                    }
                }
                flag= this.save(addTestTable);
            }

            if(flag){
                return "ok";
            }
            else{
                throw new ServiceException(ExceptionDefinition.ADD_DATA_EXCEPTION);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String editTestTable(TestTable editTestTable) throws ServiceException {
        boolean flag = this.updateById(editTestTable);
        if(flag){
            return "ok";
        }
        else{
            throw new ServiceException(ExceptionDefinition.EDIT_DATA_EXCEPTION);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String deleteTestTable(TestTable deleteTestTable) throws ServiceException {
        boolean flag = this.removeById(deleteTestTable);
        if(flag){
            return "ok";
        }
        else{
            throw new ServiceException(ExceptionDefinition.DELETE_DATA_EXCEPTION);
        }
    }

    /**
     * 每天23.55点更新(新)
     */
//    @Transactional(rollbackFor = Exception.class)
//    @Scheduled(cron = "0 55 23 * * ?")
//    public String addInfo() {
//        if ("3".equals(ENV)) {
//        }
//        return "OK";
//    }
}
